name="sorting"

from .bubble import sort
from .heap import sort
from .gnome import sort
from .counting import sort
from .comb import sort
from .cocktail import sort
from .bucket import sort
from .bogo import sort
from .shell import sort
from .selection import sort
from .radix import sort
from .quick import sort
from .merge import sort
from .insertion import sort